@@ -6,7 +6,7 @@
  * Copyright (C) 2003 Rob Adams
  * Copyright (C) 2004-2006 Elijah Newren
  * Copyright (C) 2013 Red Hat Inc.
- * Copyright (C) 2017 Alberts Muktupāvels
+ * Copyright (C) 2017-2019 Alberts Muktupāvels
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -1491,15 +1491,20 @@ gf_monitor_manager_handle_get_current_state (GfDBusDisplayConfig   *skeleton,
                                              GDBusMethodInvocation *invocation)
 {
   GfMonitorManager *manager;
+  GfMonitorManagerPrivate *priv;
+  GfSettings *settings;
   GVariantBuilder monitors_builder;
   GVariantBuilder logical_monitors_builder;
   GVariantBuilder properties_builder;
   GList *l;
   GfMonitorManagerCapability capabilities;
+  gint ui_scaling_factor;
   gint max_screen_width;
   gint max_screen_height;
 
   manager = GF_MONITOR_MANAGER (skeleton);
+  priv = gf_monitor_manager_get_instance_private (manager);
+  settings = gf_backend_get_settings (priv->backend);
 
   g_variant_builder_init (&monitors_builder, G_VARIANT_TYPE (MONITORS_FORMAT));
   g_variant_builder_init (&logical_monitors_builder, G_VARIANT_TYPE (LOGICAL_MONITORS_FORMAT));
@@ -1675,6 +1680,10 @@ gf_monitor_manager_handle_get_current_state (GfDBusDisplayConfig   *skeleton,
                              g_variant_new_boolean (TRUE));
     }
 
+  ui_scaling_factor = gf_settings_get_ui_scaling_factor (settings);
+  g_variant_builder_add (&properties_builder, "{sv}",
+                         "legacy-ui-scaling-factor",g_variant_new_int32 (ui_scaling_factor));
+
   if (gf_monitor_manager_get_max_screen_size (manager,
                                               &max_screen_width,
                                               &max_screen_height))
@@ -2196,6 +2205,12 @@ gf_monitor_manager_get_monitors (GfMonitorManager *manager)
   return manager->monitors;
 }
 
+GfLogicalMonitor *
+gf_monitor_manager_get_primary_logical_monitor (GfMonitorManager *manager)
+{
+  return manager->primary_logical_monitor;
+}
+
 GList *
 gf_monitor_manager_get_outputs (GfMonitorManager *manager)
 {
@@ -2547,31 +2562,6 @@ gf_monitor_manager_get_config_manager (GfMonitorManager *manager)
   return manager->config_manager;
 }
 
-gint
-gf_monitor_manager_get_monitor_for_output (GfMonitorManager *manager,
-                                           guint             id)
-{
-  GfOutput *output;
-  GList *l;
-
-  g_return_val_if_fail (GF_IS_MONITOR_MANAGER (manager), -1);
-  g_return_val_if_fail (id < g_list_length (manager->outputs), -1);
-
-  output = g_list_nth_data (manager->outputs, id);
-  if (!output || !output->crtc)
-    return -1;
-
-  for (l = manager->logical_monitors; l; l = l->next)
-    {
-      GfLogicalMonitor *logical_monitor = l->data;
-
-      if (gf_rectangle_contains_rect (&logical_monitor->rect, &output->crtc->rect))
-        return logical_monitor->number;
-    }
-
-  return -1;
-}
-
 gint
 gf_monitor_manager_get_monitor_for_connector (GfMonitorManager *manager,
                                               const gchar      *connector)
