@@ -177,106 +177,6 @@ flashback_monitor_labeler_new (void)
   return g_object_new (FLASHBACK_TYPE_MONITOR_LABELER, NULL);
 }
 
-void
-flashback_monitor_labeler_show (FlashbackMonitorLabeler *labeler,
-                                GfMonitorManager        *monitor_manager,
-                                const gchar             *sender,
-                                GVariant                *params)
-{
-  GVariantIter iter;
-  guint id;
-  gint number;
-  GVariant *v;
-  GHashTable *monitors;
-  GList *keys;
-  GList *key;
-  gint i;
-
-  if (labeler->hide_id > 0)
-    {
-      g_source_remove (labeler->hide_id);
-      labeler->hide_id = 0;
-    }
-
-  if (!track_client (labeler, sender))
-    return;
-
-  if (labeler->windows != NULL)
-    return;
-
-  /*if (labeler->windows != NULL)
-    {
-      for (i = 0; i < labeler->n_windows; i++)
-        gtk_widget_destroy (GTK_WIDGET (labeler->windows[i]));
-      g_free (labeler->windows);
-      labeler->windows = NULL;
-    }*/
-
-  g_variant_iter_init (&iter, params);
-
-  monitors = g_hash_table_new (g_direct_hash, g_direct_equal);
-
-  while (g_variant_iter_next (&iter, "{uv}", &id, &v))
-    {
-      gint monitor;
-
-      g_variant_get (v, "i", &number);
-
-      monitor = gf_monitor_manager_get_monitor_for_output (monitor_manager, id);
-
-      if (monitor != -1)
-        {
-          GSList *list;
-          gboolean insert;
-
-          list = (GSList *) g_hash_table_lookup (monitors, GINT_TO_POINTER (monitor));
-          insert = (list == NULL);
-
-          list = g_slist_append (list, GINT_TO_POINTER (number));
-
-          if (insert)
-            g_hash_table_insert (monitors, GINT_TO_POINTER (monitor), list);
-        }
-
-      g_variant_unref (v);
-    }
-
-  keys = g_hash_table_get_keys (monitors);
-
-  labeler->n_windows = g_hash_table_size (monitors);
-  labeler->windows = g_new0 (GfLabelWindow *, labeler->n_windows);
-  i = 0;
-
-  for (key = keys; key; key = key->next)
-    {
-      GSList *labels;
-      GSList *label;
-      GString *string;
-      gchar *real_label;
-
-      labels = (GSList *) g_hash_table_lookup (monitors, key->data);
-      string = g_string_new ("");
-
-      for (label = labels; label; label = label->next)
-        g_string_append_printf (string, "%d ", GPOINTER_TO_INT (label->data));
-      g_string_set_size (string, string->len - 1);
-
-      g_slist_free (labels);
-
-      real_label = g_string_free (string, FALSE);
-      labeler->windows[i] = gf_label_window_new (GPOINTER_TO_INT (key->data),
-                                                 real_label);
-      g_free (real_label);
-
-      gf_label_window_show (labeler->windows[i]);
-
-      i++;
-    }
-
-  g_list_free (keys);
-  g_hash_table_destroy (monitors);
-}
-
 void
 flashback_monitor_labeler_show2 (FlashbackMonitorLabeler *labeler,
                                  GfMonitorManager        *monitor_manager,
