@@ -95,7 +95,9 @@ accelerator_activated (GfKeybindings *keybindings,
 
 static gint
 real_grab (FlashbackShell *shell,
-           const gchar    *accelerator)
+           const gchar    *accelerator,
+           guint           mode_flags,
+           guint           grab_flags)
 {
   return gf_keybindings_grab (shell->keybindings, accelerator);
 }
@@ -156,12 +158,13 @@ name_vanished_handler (GDBusConnection *connection,
 static guint
 grab_accelerator (FlashbackShell *shell,
                   const gchar    *accelerator,
-                  guint           flags,
+                  guint           mode_flags,
+                  guint           grab_flags,
                   const gchar    *sender)
 {
   guint action;
 
-  action = real_grab (shell, accelerator);
+  action = real_grab (shell, accelerator, mode_flags, grab_flags);
   g_hash_table_insert (shell->grabbed_accelerators,
                        GUINT_TO_POINTER (action), g_strdup (sender));
 
@@ -212,35 +215,13 @@ handle_show_osd (FlashbackDBusShell    *dbus_shell,
 
   shell = FLASHBACK_SHELL (user_data);
 
-  flashback_osd_show (shell->osd, params);
+  flashback_osd_show (shell->osd, shell->monitor_manager, params);
 
   flashback_dbus_shell_complete_show_osd (dbus_shell, invocation);
 
   return TRUE;
 }
 
-static gboolean
-handle_show_monitor_labels (FlashbackDBusShell    *dbus_shell,
-                            GDBusMethodInvocation *invocation,
-                            GVariant              *params,
-                            gpointer               user_data)
-{
-  FlashbackShell *shell;
-  const gchar *sender;
-
-  shell = FLASHBACK_SHELL (user_data);
-  sender = g_dbus_method_invocation_get_sender (invocation);
-
-  g_assert (shell->monitor_manager != NULL);
-
-  flashback_monitor_labeler_show (shell->labeler, shell->monitor_manager,
-                                  sender, params);
-
-  flashback_dbus_shell_complete_show_monitor_labels (dbus_shell, invocation);
-
-  return TRUE;
-}
-
 static gboolean
 handle_show_monitor_labels2 (FlashbackDBusShell    *dbus_shell,
                              GDBusMethodInvocation *invocation,
@@ -306,7 +287,8 @@ static gboolean
 handle_grab_accelerator (FlashbackDBusShell    *dbus_shell,
                          GDBusMethodInvocation *invocation,
                          const gchar           *accelerator,
-                         guint                  flags,
+                         guint                  mode_flags,
+                         guint                  grab_flags,
                          gpointer               user_data)
 {
   FlashbackShell *shell;
@@ -315,7 +297,7 @@ handle_grab_accelerator (FlashbackDBusShell    *dbus_shell,
 
   shell = FLASHBACK_SHELL (user_data);
   sender = g_dbus_method_invocation_get_sender (invocation);
-  action = grab_accelerator (shell, accelerator, flags, sender);
+  action = grab_accelerator (shell, accelerator, mode_flags, grab_flags, sender);
 
   flashback_dbus_shell_complete_grab_accelerator (dbus_shell, invocation, action);
 
@@ -344,12 +326,13 @@ handle_grab_accelerators (FlashbackDBusShell    *dbus_shell,
   while ((child = g_variant_iter_next_value (&iter)))
     {
       gchar *accelerator;
-      guint flags;
+      guint mode_flags;
+      guint grab_flags;
       guint action;
 
-      g_variant_get (child, "(su)", &accelerator, &flags);
+      g_variant_get (child, "(suu)", &accelerator, &mode_flags, &grab_flags);
 
-      action = grab_accelerator (shell, accelerator, flags, sender);
+      action = grab_accelerator (shell, accelerator, mode_flags, grab_flags, sender);
       g_variant_builder_add (&builder, "u", action);
 
       g_free (accelerator);
@@ -377,7 +360,7 @@ handle_ungrab_accelerator (FlashbackDBusShell    *dbus_shell,
   sender = (gchar *) g_hash_table_lookup (shell->grabbed_accelerators,
                                           GUINT_TO_POINTER (action));
 
-  if (g_str_equal (sender, g_dbus_method_invocation_get_sender (invocation)))
+  if (g_strcmp0 (sender, g_dbus_method_invocation_get_sender (invocation)) == 0)
     {
       success = real_ungrab (shell, action);
 
@@ -409,8 +392,6 @@ name_appeared_handler (GDBusConnection *connection,
                     G_CALLBACK (handle_focus_search), shell);
   g_signal_connect (skeleton, "handle-show-osd",
                     G_CALLBACK (handle_show_osd), shell);
-  g_signal_connect (skeleton, "handle-show-monitor-labels",
-                    G_CALLBACK (handle_show_monitor_labels), shell);
   g_signal_connect (skeleton, "handle-show-monitor-labels2",
                     G_CALLBACK (handle_show_monitor_labels2), shell);
   g_signal_connect (skeleton, "handle-hide-monitor-labels",
